// - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// ZOMBIE TORRENT 2 - GAMEMODE
// Contains game-related things
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - 

// Basic state machine for the game

enum GameStateType
{
	STATE_LOBBY,				// Game is in the lobby, waiting for players to be ready
	STATE_COUNTDOWN,			// Next wave is starting, we're doing the countdown
	STATE_WAVE,					// The wave has started! Zombies, gameplay, etc.
	STATE_POSTWAVE,				// All zombies are killed, do a short delay to tradertime
	STATE_TRADER,				// We're in tradertime
	STATE_ENDGAME,				// Boss has killed us, or we won!
};

// Holder for gamestate variables
// This keeps it compressed, rather than having a ton of them

struct GameStateStruct
{
	int State;							// Current state for the game
	int Wave;							// Which wave are we on? (0-based)
	int ZombiePool;						// We can only spawn zombies if this is > 0
	int WaveTimer;						// Wave countdown / trader time
	int ZombiesLeft;					// This is the number shown on the HUD
	int CurrentTrader;					// Which trader in the array are we using?
};

struct GameStateStruct GameState;

// Holder for player variables

struct PlayerStateStruct
{
	bool Ready;							// Is this player ready?
	bool Alive;							// This player is currently "alive"
	bool InTrader;						// This player is in the trader
	int TraderIndex;					// WHICH trader are we inside of?
};

struct PlayerStateStruct PlayerState[MAXPLAYERS];

// -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

script "GameInitServer" OPEN { ZT2Initialize(true); }
script "GameInitClient" OPEN CLIENTSIDE { ZT2Initialize(false); }

// Initialize core game elements!
void ZT2Initialize(int isServer)
{
	RegisterZT2Weapons(isServer);
	RegisterZT2Zombies(isServer);
}

// -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

// Can the game be IMMEDIATELY joined?
bool GameIsJoinable()
{
	return (GameState.State == STATE_TRADER || GameState.State == STATE_COUNTDOWN);
}

// -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

// ------------------------/
// From the lobby, start the actual game!
// ------------------------/

void StartInvasionGame()
{
	// First wave
	GameState.Wave = 0;
	RevivePlayers();
	
	BeginWave();
}

// ------------------------/
// Actually start a wave!
// ------------------------/

void BeginWave()
{	
	GameState.State = STATE_COUNTDOWN;
	RevivePlayers();
	ACS_NamedExecute("WaveCountdown",0,0,0,0);
}

// ------------------------/
// Countdown script for the wave!
// ------------------------/

script "WaveCountdown" (void)
{
	GameState.WaveTimer = COUNTDOWN_TIME;
	
	while (GameState.WaveTimer > 0)
	{
		PrintBold(s:"WAVE ", d:GameState.Wave+1, s:" IS STARTING IN ", d:GameState.WaveTimer, s:" SECONDS...");
		delay(35);
		GameState.WaveTimer --;
	}
	
	BeginWaveProcess();
}

// ------------------------/
// Begin a wave's PROCESS, this is post-countdown!
// ------------------------/

void BeginWaveProcess()
{
	GameState.ZombiePool = ZOMBIEPOOL_BASE;
	GameState.State = STATE_WAVE;
	
	UpdateZombiesLeft();
	
	PrintBold(s:"\cGFIGHT!!!");
	ACS_NamedExecute("SpawnDirectorAI",0,0,0,0);
}

// ------------------------/
// Update the VISUAL number of zombies left
// ------------------------/

void UpdateZombiesLeft(bool Calculate = true)
{
	if (Calculate)
		GameState.ZombiesLeft = GameState.ZombiePool + AliveZombies();
		
	// All zombies have been killed!
	if (AllZombiesKilled())
		CompleteWave();
}

// ------------------------/
// Go to the next wave
// ------------------------/

void IncreaseWave()
{
	GameState.Wave ++;
}

// ------------------------/
// Complete a wave! All zombies were killed!
// ------------------------/

void CompleteWave()
{
	KillSpawnDirector();
	GameState.State = STATE_POSTWAVE;
	ACS_NamedExecuteAlways("PostWaveDelay",0,0,0,0);
}

// ------------------------/
// Do a slight delay before trader time
// ------------------------/

script "PostWaveDelay" (void)
{
	Delay(35*2);
	BeginTraderTime();
}

// ------------------------/
// Actually begin trader time!
// ------------------------/

void BeginTraderTime()
{
	GameState.State = STATE_TRADER;
	PrintBold(s:"TRADER IS OPEN!!!!");
	
	GameState.CurrentTrader = ChooseTrader();
	
	OpenTraderDoor();
	
	GameState.WaveTimer = TRADER_DEFAULTTIME;
	
	ACS_NamedExecute("TraderTimerScript",0,0,0,0);
}

// ------------------------/
// Responsible for controlling trader time
// ------------------------/

script "TraderTimerScript"
{
	while (GameState.WaveTimer > 0)
	{
		PrintBold(s:"TRADER SECONDS LEFT: ", d:GameState.WaveTimer);
		delay(35);
		GameState.WaveTimer --;
	}
	
	CloseTraderDoor();
	TeleportPlayers();
	
	IncreaseWave();
	BeginWave();
}

// ------------------------/
// Open the trader doors
// ------------------------/

void OpenTraderDoor()
{
	int doorID = TraderArray[GameState.CurrentTrader].DoorID;
	Door_Open(doorID, 64);
}

// ------------------------/
// Close the trader doors
// ------------------------/

void CloseTraderDoor()
{
	int doorID = TraderArray[GameState.CurrentTrader].DoorID;
	Door_Close(doorID, 64);
}

// ------------------------/
// Enter / exit the trader
// ------------------------/

script SCRIPT_ENTERTRADER (int index) { EnterTrader(index); }
void EnterTrader(int tIndex)
{
	int PN = PlayerNumber();
	int LS = LineSide();
	
	// Enter from front
	if (LS == 0)
	{
		PlayerState[PN].InTrader = true;
		PlayerState[PN].TraderIndex = tIndex;
	}
		
	// Exit from back
	else
	{
		PlayerState[PN].InTrader = false;
		PlayerState[PN].TraderIndex = -1;
	}
}

// ------------------------/
// Teleport all players out of the trader!
// ------------------------/

void TeleportPlayers()
{
	int OID = ActivatorTID();
	int destSpot = TraderArray[GameState.CurrentTrader].SpotID;
	
	for (int i=0; i<MAXPLAYERS; i++)
	{
		if (PlayerState[i].InTrader && PlayerInGame(i))
		{
			SetActivator(i+BASE_PLAYER_TID);
			Teleport(destSpot, 0, 0);
		}
			
		PlayerState[i].InTrader = false;
	}
	
	SetActivator(OID);
}

// ------------------------/
// Forcefully end the wave! 
// This is a debug command
// ------------------------/

script "ForceEndWave" (void) NET { ForceEndWave(); }
void ForceEndWave()
{
	// Mid-wave? Kill all zombies
	if (GameState.State == STATE_WAVE)
	{
		GameState.ZombiePool = 0;
		GameState.ZombiesLeft = 0;
		KillZombies();
	}
	
	// Countdown, just increase
	else if (GameState.State == STATE_COUNTDOWN)
	{
		IncreaseWave();
		BeginWave();
	}
}